/*
*  AddressDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "AddressDlgs.h"
#include "AddrEditDlgs.h"
#include "HelpDlgs.h"
#include "MsgWins.h"
#include "FileMan.h"
#include "HeadMan.h"
#include "Global.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TAddressDlg *AddressDlg;
//---------------------------------------------------------------------------
bool __fastcall GetMailRecipients(TMessageWin *Msgwin)
{
   bool got=false;
   TAddressDlg *AddressDlg = new TAddressDlg(Msgwin);
   if (AddressDlg->Execute()) got=true;
   delete AddressDlg;
   return got;
}
//---------------------------------------------------------------------------
// Address Dialog
//---------------------------------------------------------------------------
__fastcall TAddressDlg::TAddressDlg(TComponent* Owner)
	: TForm(Owner)
{
   CenterDialog(this,AppMainForm);

   // initialize Address list
   GetAppData(AddrsList->Items,UserAddresses+PS+General);
   // initialize Newsgroups list
   GetAppData(GrpsList->Items,UserAddresses+PS+Newsgroups);
   // initialize stuff
   Msgwin = dynamic_cast<TMessageWin*>(Owner);
   if (!Msgwin) return;

   Items = new TStringList();
   Ed = Msgwin->Editor;
   Headman = new THeaderManager(Ed->Lines);
   OrgHeaders = new TStringList();
   for (int i=0; i<Ed->Lines->Count; i++)
   {
      AnsiString hdr = Ed->Lines->Strings[i];
      if (hdr.Trim().IsEmpty()) break;
      OrgHeaders->Add(hdr);
   }

   PageControl1->ActivePage = TabSheet1;
   Header2ListBox("To: ",AddrsList);
   ToBtn->Checked = true;
   Modified = false;
}
//---------------------------------------------------------------------------
__fastcall TAddressDlg::~TAddressDlg()
{
   delete OrgHeaders;
   delete Items;
}
//---------------------------------------------------------------------------
bool __fastcall TAddressDlg::DoingNewsgroups()
{
   return PageControl1->ActivePage==TabSheet2;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAddressDlg::ChkdHeader()
{
   if (DoingNewsgroups()) return "Newsgroups: ";
   if (BcBtn->Checked) return "Bcc: ";
   if (CcBtn->Checked) return "Cc: ";
   return "To: ";
}
//---------------------------------------------------------------------------
bool __fastcall TAddressDlg::Execute()
{
   AppProcessMessages();
   ShowModal();
   return ModalResult==mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::EditAddrBtnClick(TObject *Sender)
{
   if (EditAddressesEtc(AddrsList->Items,NULL,this))
      PutAppData(UserAddresses+PS+General,AddrsList->Items);
   Header2ListBox(ChkdHeader(),AddrsList);
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::EditGrpsBtnClick(TObject *Sender)
{
   if (EditAddressesEtc(NULL,GrpsList->Items,this))
      PutAppData(UserAddresses+PS+Newsgroups,GrpsList->Items);
   Header2ListBox(ChkdHeader(),GrpsList);
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::ListBoxDblClick(TObject *Sender)
{
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::HelpBtnClick(TObject *Sender)
{
   OpenHelp("AddressBook","Address Book","",(TForm*)Owner);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAddressDlg::GetHeaderCommaText(AnsiString header)
{
   int pos; AnsiString msgheader = Headman->GetDat(header);
   while ((pos=msgheader.Pos("\r"))>0) msgheader.Delete(pos,1);
   while ((pos=msgheader.Pos("\t"))>0) msgheader.Delete(pos,1);
   while ((pos=msgheader.Pos(", "))>0) msgheader.Delete(pos+1,1);
   while ((pos=msgheader.Pos(" ,"))>0) msgheader.Delete(pos,1);
   return msgheader;
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::GetHeaderItems(AnsiString header, TStrings *items)
{
   items->Clear();
   int pos=0; AnsiString line, headertxt = GetHeaderCommaText(ChkdHeader());
   while ((line = GetToken(pos++, headertxt, ",")) != "")
      items->Add(line.Trim());
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::Header2ListBox(AnsiString header, TListBox *list)
{
   ClearSelected(list);
   GetHeaderItems(header,Items);
   for (int i=0; i<Items->Count; i++)
   {
      int idx; AnsiString line = Items->Strings[i];
      if ((idx=list->Items->IndexOf(line))>=0) list->Selected[idx] = true;
      else
      {
         list->Items->Add(line);//.Trim());
         list->Selected[list->Items->IndexOf(line)];
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::ClearSelected(TListBox *list)
{
   for (int i=0; i<list->Items->Count; i++)
      list->Selected[i] = false;
}
//---------------------------------------------------------------------------

void __fastcall TAddressDlg::ListBoxClick(TObject *Sender)
{
   TListBox *List = DoingNewsgroups() ? GrpsList : AddrsList;
   AnsiString addrs, header = ChkdHeader();
   GetHeaderItems(header, Items);
   for (int i=0; i<List->Items->Count; i++)
   {
      addrs = List->Items->Strings[i];
      int idx = Items->IndexOf(addrs);
      if (idx >= 0 && !List->Selected[i])
         Items->Delete(idx);
      else
      if (idx < 0 && List->Selected[i])
         Items->Add(addrs);
   }

   CancelBtn->Caption = "Cancel";
   ReplaceHeader(header, Items);
}
//---------------------------------------------------------------------------

void __fastcall TAddressDlg::ReplaceHeader(AnsiString header, TStrings *lines)
{
   SetRedraw(Msgwin,0);
   Ed->Lines->BeginUpdate();
   HeadMan->HereIs(Ed->Lines);

   int idx = Headman->Delete(header);
   if (idx < 0 && header == "Newsgroups: " && HeadMan->Has("Post: "))
   {
      idx = Headman->Delete("Post: ");
      header = "Post: ";
   }

   if (idx < 0)
      idx = Headman->Eoh();

   for (int i=0; i<Items->Count; i++)
   {
      AnsiString line = Items->Strings[i];
      if (i==0) line = header+line;
      else line = " "+line;

      if (i<Items->Count-1) line += ",";
      Ed->Lines->Insert(idx+i,line);
   }

   Ed->Lines->EndUpdate();
   AppProcessMessages();
   SetRedraw(Msgwin,1);
   Modified = true;
}
//---------------------------------------------------------------------------

/*void __fastcall TAddressDlg::ReplaceHeader(AnsiString header, TStrings *lines)
{
   SetRedraw(Msgwin,0);
   Ed->Lines->BeginUpdate();
   int idx = Headman->Delete(header);
   if (idx < 0)
      idx = Headman->Eoh();

   for (int i=0; i<Items->Count; i++)
   {
      AnsiString line = Items->Strings[i];
      if (i==0) line = header+line;
      else line = " "+line;

      if (i<Items->Count-1) line += ",";
      Ed->Lines->Insert(idx+i,line);
   }

   Ed->Lines->EndUpdate();
   AppProcessMessages();
   SetRedraw(Msgwin,1);
   Modified = true;
}*/
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::HdrBtnClick(TObject *Sender)
{
   AnsiString header;
   if (Sender==GroupsBtn) header = "Newsgroups: ";
   if (Sender==BcBtn) header = "Bcc: ";
   if (Sender==CcBtn) header = "Cc: ";
   if (Sender==ToBtn) header = "To: ";
   TListBox *list = DoingNewsgroups()?GrpsList:AddrsList;
   Header2ListBox(header,list);
}
//---------------------------------------------------------------------------
void __fastcall TAddressDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult==mrCancel && Modified)
   {
      switch (MsgBoxHwnd(Handle,
         "Your headers have been modified.\n\nKeep your changes?","Confirm",
          MB_YESNOCANCEL))
      {
         case IDNO:
            Cursor = crHourGlass;
            while (Ed->Lines->Count && !Ed->Lines->Strings[0].Trim().IsEmpty())
               Ed->Lines->Delete(0);
            OrgHeaders->AddStrings(Ed->Lines);
            Ed->Lines->Text = OrgHeaders->Text;
            AppProcessMessages(); Sleep(200);
            CancelBtn->Caption = "Close";
            Cursor = crDefault;
            break;
         case IDCANCEL:
            CanClose = false;
            break;
      }
   }
   Ed->SelStart = 0;
}
//---------------------------------------------------------------------------

