/*
*  MsgTooBigDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "global.h"
#include "MsgTooBigDlgs.h"
#include "Fileman.h"
#include "mixmaster.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TMessageTooBigDlg *MessageTooBigDlg;

char errormsg1[] = "\n\
I'm sorry, your message is too large for the exit remailer of your chain.\n\n";

char errormsg2[] = "\n\
I'm sorry, your message is rather large and your exit remailer is not currently listed \
in the caps strings portion of rlist.txt or mlist.txt. That means I can't check to see if that remailer \
will accept your message. You can continue, but I suggest you read the instructions below, \
then cancel the send, and select a final remailer which will surely accept this message.\n\n";

char errormsg3[] = "\
You will need to close this dialog and open the remailer document 'rlist.txt'. On the \
QS main menu click 'View | rlist.txt'. In the section of rlist where entries begin with \
'$remailer' find a remailer with both 'mix' capability and either no 'klen' present or a \
'klen' large enough to accept your message. It's quite easy to determine even if you are \
inexperienced. If you find a suitable remailer, you need to change your chain header \
by specifically placing this remailer's name as last in your chain. NOTE: Remember klen \
is given in thousands ok bytes, so 'klen1024' is 1,024,000 bytes.\n\n\
The mixmaster protocol is not a good protocol for large messages/attachments. All mixmaster \
messages are 29K in size. If your message will not fit in 29K then it must be divided \
into a series of smaller messages. The biggest problem is that, of course, if even 1 \
of those parts fails to make it to the last remailer, your message will not be delivered. \
As the number of divisions goes up, reliability of delivery goes down. Keep this in mind \
when thinking about attaching large files.";

//---------------------------------------------------------------------------

bool __fastcall CheckExitRemailerKlen(TForm *Owner, AnsiString chain, int msgsize)
{
   //AnsiString largeish;
   //GetAppIni(LargeishSize, largeish);
   //int biggie = largeish.ToIntDef(-1);
   //if (biggie < 0) biggie = InitialLargeishSize;
   //if (biggie < MinLargeishSize) biggie = MinLargeishSize;

   // check mlist.txt first for caps strings then, if none check rlist
   TStringList *rlist = GetRlistCapsStrings(mlisttxt);
   if (rlist->Count == 0)
   {
      delete rlist;
      rlist = GetRlistCapsStrings(clisttxt);
   }
   
   AnsiString exitrem = GetExitRemailer(chain);
   int klen = GetRlistRemailerKlen(rlist, exitrem);
   delete rlist;

   if (klen > 0 && msgsize > klen)
   {
      MessageTooBigDlg = new TMessageTooBigDlg(Owner, exitrem, msgsize, klen);
      MessageTooBigDlg->ShowModal();
      delete MessageTooBigDlg;
      return false;
   }

   if (klen < 0 && msgsize > InitialLargeishSize)
   {
      MessageTooBigDlg = new TMessageTooBigDlg(Owner, exitrem, msgsize);
      MessageTooBigDlg->ShowModal();
      bool ok = MessageTooBigDlg->ModalResult == mrOk;
      delete MessageTooBigDlg;
      return ok;
   }
   return true;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetExitRemailer(AnsiString &chain)
{
   AnsiString exitrem = chain;
   if (chain.Pos(",") == 0) return chain;

   int i = chain.Length();
   while (i > 0)
   {
      if (chain[i] == ',')
      {
         exitrem = chain.SubString(i + 1, 16);
         break;
      }
      i--;
   }

   return exitrem;
}
//---------------------------------------------------------------------------

TStringList* __fastcall GetRlistCapsStrings(const AnsiString &rlisttxt)
{
   TStringList *rlist = new TStringList();

   if (FileExists(rlisttxt))
      rlist->LoadFromFile(rlisttxt);

   int i = 0; while (i < rlist->Count)
   {
      if (rlist->Strings[i].Pos("$remailer{") == 1) i++;
      else rlist->Delete(i);
   }

//view(rlist, rlisttxt);
   return rlist;
}
//---------------------------------------------------------------------------

bool __fastcall CheckRlistIsCpunkOnly(TStrings *rlist)
{
   bool hasmixonlyrems = false;

   for (int i = 0; i < rlist->Count; i++)
   {
      if (rlist->Strings[i].Pos(" cpunk ") == 0)
      {
         hasmixonlyrems = true;
         break;
      }
   }

   return hasmixonlyrems;
}
//---------------------------------------------------------------------------

int __fastcall GetRlistRemailerKlen(TStrings *rlist, const AnsiString &remailer)
{
   int klen = -1;

   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString field, line = rlist->Strings[i];
      if (line.Pos("$rem") == 1 && GetToken(1, line, "\"") == remailer)
      {
         klen = 0;
         line = GetToken(3, line, "\"");
         int x = 0; while ((field = GetToken(x, line, " ")) != "")
         {
            if (field.Pos("klen") == 1)
            {
               klen = field.SubString(5, field.Length()).ToIntDef(0);
               break;
            }
            x++;
         }
         break;
      }
   }

   if (klen > 0) return klen * 1000;

   return klen;
}
//---------------------------------------------------------------------------

/*void __fastcall MessageTooBigError(TForm *Owner, const AnsiString &Remailer,
   int MsgSize, int MaxSize)
{
   MessageTooBigDlg = new TMessageTooBigDlg(Owner, Remailer, MsgSize, MaxSize);
   MessageTooBigDlg->ShowModal();
   delete MessageTooBigDlg;
}*/
//---------------------------------------------------------------------------

__fastcall TMessageTooBigDlg::TMessageTooBigDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

AnsiString __fastcall WithCommas(int number)
{
   AnsiString numstr = IntToStr(number);

   if (numstr.Length() > 3)
      numstr.Insert(",", numstr.Length() - 2);

   if (numstr.Length() > 7)
      numstr.Insert(",", numstr.Length() - 6);

   return numstr;
}
//---------------------------------------------------------------------------

__fastcall TMessageTooBigDlg::TMessageTooBigDlg(TComponent* Owner, const AnsiString &Remailer,
   int MsgSize, int MaxSize) : TForm(Owner)
{
   CenterDialog(this, Owner);

   Label2->Caption = Remailer + " max = " + WithCommas(MaxSize) +
      " bytes,  your message = " + WithCommas(MsgSize) + " bytes";

   OkBtn->Left = CancelBtn->Left;
   CancelBtn->Hide();

   RichEdit1->Lines->Text = errormsg1;
   RichEdit1->SelText = errormsg3;
}
//---------------------------------------------------------------------------

__fastcall TMessageTooBigDlg::TMessageTooBigDlg(TComponent* Owner, const AnsiString &Remailer,
   int MsgSize) : TForm(Owner)
{
   CenterDialog(this, Owner);

   Label1->Caption = "Large Message Caution";
   Label2->Caption = Remailer + " max =  ?  bytes, your message = " +
      WithCommas(MsgSize) + " bytes";
   OkBtn->Caption = "Continue";
   Caption = "Problem Report";

   RichEdit1->Lines->Text = errormsg2;
   RichEdit1->SelText = errormsg3;
}
//---------------------------------------------------------------------------
void __fastcall TMessageTooBigDlg::PrintBtnClick(TObject *Sender)
{
   if (MsgDlg(this, "Print these instructions?", "Confirm", MB_OKCANCEL) == IDOK)
      RichEdit1->Print("Klen Instructions");
}
//---------------------------------------------------------------------------
